import bpy
import os
import sys
import datetime
import urllib
import codecs



mode = "CGto3DPrint"
currentPath = bpy.path.abspath("//")
scriptFolder = "C:\CGto3DPrint\\"
scriptFileName = "CGto3DPrint.bat"
#scriptFileName = "CGto3DPrint0.2.bat"
#scriptFileName = "CGto3DPrint0.5.bat"
defaultObject = bpy.context.active_object


def cgto3dprint():

    # Output information in console
    print('\n\n')
    d = datetime.datetime.today()    
    print("--- CGto3DPrint ---")
    print("by Lunavast")
    print(d.strftime("%Y-%m-%d %H:%M:%S"), '\n')
         
    # get the active object
    obj = bpy.context.active_object
    
    # Delete Mesh object
    """
    obj002name = defaultObject.name + ".002"
    
    if existObj(obj002name) == True:
        obj002 = bpy.data.objects[obj002name]
        clearMesh(obj002)
    """
    obj001name = defaultObject.name + ".001"
    
    if existObj(obj001name) == True:
        obj001 = bpy.data.objects[obj001name]
        clearMesh(obj001)
    
    # Export active object mesh to STL file
    exportMeshStl(obj)
    
    # Run Meshlab bat file
    batFile = scriptFolder + 'scripts\\' + scriptFileName
    runBat(batFile)
    print("batFile: ", batFile)
    #runBat(batFile,' ',param[6],'meshlabserver.exe')
    
    # Import STL file
    importMeshStl(obj)
    
    # Delete used files
    inputPath = scriptFolder + "input\\" + obj.name + ".stl"
    outputPath = scriptFolder + "output\\" + obj.name + ".stl"
    os.remove(inputPath)
    os.remove(outputPath)
        
    # selectLinkedSeparate
    selectLinkedSeparate()
    
    # Modifier Decimate
    #obj001name = defaultObject.name + ".001"
    obj001 = bpy.data.objects[obj001name]

    type = 'Decimate'
    #ratio = 0.1
    
    ratio = importConfigDecimate(scriptFolder + 'config.ini')
    #ratio = importConfigDecimate(scriptFolder + 'config.ini')
    modifierDecimate(type, obj001, ratio)
    

    # Texture Baking
    # Reference: http://junnno.sakura.ne.jp/3d/n3.1/    
    
    ## Smart UV Project
    ### http://blender.stackexchange.com/questions/17738/how-to-uv-unwrap-object-with-python
    
    ### Creating an image file for uv map
    #obj001name = "Subaru360mesh.001"
    #obj001 = bpy.data.objects[obj001name]

    #createImage(obj001)
    
    ### Smart UV Project
    print("Smart UV Project")

    imageFilePath = currentPath + obj001.name + ".png"
    
    #textureMaterial(obj001, imageFilePath)
    Mapper()
    
    # Create an image
    #createImage(obj001)
    
            
    ### Baking original texture to unwrapped texture
    print("Texture copy") 
    originalObjName = defaultObject.name
    originalObj = bpy.data.objects[originalObjName]

    #obj001name = "Subaru360mesh" + ".001"
    obj001 = bpy.data.objects[obj001name]
   
    bakeTextureCopy(obj001, originalObj)
    
    ### Baking unwrapped texture to vertex color
    bakeTextureToVcols()
    print("Job completed") 
    
    
def bakeTextureCopy(obj001, originalObj):

    #bpy.ops.object.mode_set(mode = 'OBJECT')
    
    # Object selection
    originalObj.select = True
    obj001.select = True
        
    # Baking settings
    bpy.data.scenes["Scene"].render.bake_type = "TEXTURE"
    bpy.data.scenes["Scene"].render.use_bake_to_vertex_color = False
    bpy.data.scenes["Scene"].render.use_bake_clear = True
    bpy.data.scenes["Scene"].render.bake_margin = 2
    bpy.data.scenes["Scene"].render.use_bake_selected_to_active = True
    bpy.data.scenes["Scene"].render.bake_distance = 1.0
    bpy.data.scenes["Scene"].render.bake_bias = 0.001
   
    # Baking
    bpy.ops.object.bake_image()
    originalObj.select = False   
    
    
def bakeTextureToVcols():
    bpy.data.scenes["Scene"].uv_bake_alpha_color = (1.0, 1.0, 1.0)
    bpy.ops.uv.bake_texture_to_vcols()
    

def clearMesh(obj):
    if obj.type == 'MESH':
        mesh = obj.data
        bpy.context.scene.objects.unlink(obj)	
        bpy.data.objects.remove(obj)
        bpy.data.meshes.remove(mesh)

def existObj(objName):
    if bpy.data.objects.get(objName) is not None:
        return True
    else:
        return False
 
 
def exportMeshStl(obj):
    # Export active object mesh to STL file
    #print("Active object: ", obj.name)
    
    exportStlFile = scriptFolder + "input\\" + obj.name + ".stl"
    bpy.ops.export_mesh.stl(filepath = exportStlFile)
    print("File exported: ", exportStlFile)

    
def importMeshStl(obj):
    # Export active object mesh to STL file
    #print("Active object: ", obj.name)
    
    importStlFile = scriptFolder + "output\\" + obj.name + ".stl"
    bpy.ops.import_mesh.stl(filepath = importStlFile)
    print("File imported: ", importStlFile)


def importConfigDecimate(filePath):
    #print("filePath: ", filePath)
    f = codecs.open(filePath, 'r','utf-8')
    #f = open(filePath, 'r')

    i = 0
    
    for l in f:
        l = l.rstrip()
        #print("l: ", l)
        #print("f: ", f)

        if (i == 2):
            decimatevalue = float(l)        
            return decimatevalue
        
        i += 1
        #print("i : ", i)
       
    f.close()
    

def modifierDecimate(type, obj, ratio):
    print("ratio : ", ratio)
    #obj = context.active_object
    if type == 'Decimate':
        dm = obj.modifiers.new('Decimate','DECIMATE')
    dm.ratio = ratio
    
    bpy.context.scene.objects.active = obj
    
    print("Modifier:", dm)
    bpy.ops.object.modifier_apply(modifier="dm")


def runBat(batFile):
    # Run Meshlab bat file
    os.system(batFile)

def selectLinkedSeparate():
    # http://blender.stackexchange.com/questions/27376/python-script-for-a-specific-selection
    
    obj = bpy.context.object
    
    
    bpy.ops.object.mode_set(mode = 'EDIT')
    bpy.ops.mesh.select_all(action ='DESELECT')
    max = -9999
    for i in obj.data.vertices :
        if i.co[2] > max :
            max_ind = i.index
            max  = i.co[2]
    bpy.ops.object.mode_set(mode = 'OBJECT')
    obj.data.vertices[max_ind].select = True
    bpy.ops.object.mode_set(mode = 'EDIT')
    bpy.ops.mesh.select_linked()
    bpy.ops.mesh.separate(type='SELECTED')
    bpy.ops.object.mode_set(mode = 'OBJECT')
    
    
    # STL Export
    path = bpy.path.abspath("//")
    #obj2name = obj.name + ".001"
    
    obj2name = defaultObject.name + ".002"
    
    #if bpy.data.objects.get(obj2name) is not None
    if existObj(obj2name) == True:
        obj2 = bpy.data.objects[obj2name]
    
        # Rename
        obj2.name = defaultObject.name + ".001"

    # Shrink the model
    #obj001name = obj.name + ".001" 
    #obj001name = obj2.name 
    obj001name = defaultObject.name + ".001" 
    obj001 = bpy.data.objects[obj001name]
    #targetObj = obj
    targetObj = defaultObject
    #modifierShrinkwrap(obj001, targetObj)
   
    
    fileName = obj2.name#"output"#obj.name
    stlPath = path + fileName + ".stl"
    obj.select = False
    bpy.ops.export_mesh.stl(filepath=stlPath)
    
    # Oblect delete
    obj.select = True
    obj2.select = False
    bpy.ops.object.delete()
    obj2.select = True
    
    #print('obj2:', obj2)
    #print('stlPath:', stlPath)
    

def Mapper():
    act = bpy.context.active_object
    bpy.ops.object.editmode_toggle()
    bpy.ops.mesh.select_all(action='SELECT')
    bpy.ops.uv.smart_project()
    bpy.ops.uv.seams_from_islands()
    bpy.ops.uv.unwrap(method='ANGLE_BASED', margin=0.001)
    bpy.ops.uv.pack_islands(margin=0.001)
    
    bpy.ops.image.new(name='UVimage', width = 1024, height = 1024, )
    bpy.data.screens['UV Editing'].areas[1].spaces[0].image = bpy.data.images["UVimage"] # Set the new image to the new UV texture 

    bpy.ops.object.editmode_toggle()
    
    

if __name__ == "__main__":

    #param = sys.argv
    #print param
    #print ("param[1]: ",param[1])

    #ratio = importConfigDecimate(scriptFolder + 'config.ini')
    #print ("ratio: ",ratio)
        
    if mode == "CGto3DPrint":
        cgto3dprint()